####Packages and data #####
library(lme4)
library(reshape)#for merge
library(car)
library(MCMCglmm)
library(boot)
library(pedantics)

AllM<-read.table(file = "AllM.txt",header=TRUE)
ped <- read.table(file = "ped.txt", header=TRUE)
ped <- orderPed(ped)

#####Quantitative genetics####
#### Blups analysis ####
mmass0<-MCMCglmm(Weight ~ 1 + f+Sex*Age*StDate+StDateSq,
                 random =~ animal + id + Year + Mother, 
                 rcov=~units, prior = priorfitness0, data=AllM, family="gaussian",pedigree=ped,
                 verbose=TRUE,nitt=120000,burnin=20000,thin=100,pr = TRUE)
summary(mmass0)
h2mmass0<-mmass0$VCV[,"animal"]/(rowSums(mmass0$VCV[,Gindex]))
plot(h2mmass0)
posterior.mode(h2mmass0);mean(h2mmass0);HPDinterval(h2mmass0)

BV<-mmass0$Sol[,grep(pattern = "animal*",x = colnames(mmass0$Sol))]

namesBV<- substr(dimnames(BV)[[2]],start = 8,stop = nchar(dimnames(BV)[[2]]))
BV<-BV[,which(namesBV %in% AllM$id)]

animalID<-substr(x = colnames(BV),start = 8,stop=nchar(colnames(BV)))
pmBV<-data.frame(animalID,posterior.mode(BV))
names(pmBV)<-c("id","pBV")
plot(pmBV)
meanY<-data.frame(tapply(AllM$Year-2006,AllM$id,mean),names(tapply(AllM$Year,AllM$id,mean)))
meanY<-data.frame(tapply(AllM$Cohort-2006,AllM$id,mean),names(tapply(AllM$Cohort,AllM$id,mean)))
names(meanY)<-c("Year","id")
mpmBV<-merge(x = pmBV,y = meanY,by = "id",all.x=TRUE ,all.y = FALSE)
plot(tapply(mpmBV$pBV,mpmBV$Year,mean),x=as.numeric(names(tapply(mpmBV$pBV,mpmBV$Year,mean))))

length(lmBV)
lmBV<-as.mcmc(apply(BV,MARGIN = 1,function(x){coef(lm(x~1+mpmBV$Year))[2]}))
plot(lmBV)
posterior.mode(lmBV);HPDinterval(lmBV)
sum(lmBV>0)/length(lmBV)

#### Main model for Robertson-Price equation ####
priorgBiv1<-list(G=list(G1=list(V=diag(2), nu=2, alpha.mu=rep(0,2), alpha.V=diag(2)*1000),
                        G2=list(V=diag(2), nu=2, alpha.mu=rep(0,2), alpha.V=diag(2)*1000),
                        G3=list(V=diag(2), nu=2, alpha.mu=rep(0,2), alpha.V=diag(2)*1000),
                        G4=list(V=diag(2), nu=2, alpha.mu=rep(0,2), alpha.V=diag(2)*1000)),
                 R=list(V=diag(2), nu=2))

mgBivLRS2<-MCMCglmm(cbind(Weight,rLRS2) ~ trait-1 + trait:(f)+at.level(trait,1):(Sex*Age*StDate+StDateSq),
                    random =~ us(trait):animal + us(trait):id + us(trait):Mother + idh(trait):Year, 
                    rcov=~us(trait):units, prior = priorgBiv1, data=AllM,pedigree=ped, family=c("gaussian","gaussian"),
                    verbose=TRUE,nitt=3000000,burnin=1000000,thin=1000)
summary(mgBivLRS2)

#mode and credibility interval of the additive genetic covariance mass-fitness
posterior.mode(mgBivLRS2$VCV[,"Weight:rLRS2.animal"]) ; HPDinterval(mgBivLRS2$VCV[,"Weight:rLRS2.animal"])

#### Full model with all traits (long to run, good to confirm by pair-wise bivariate models like the one above) ####
priorg0<-list(G=list(G1=list(V=diag(4), nu=4, alpha.mu=rep(0,4), alpha.V=VA*1000),
                     G2=list(V=diag(4), nu=4, alpha.mu=rep(0,4), alpha.V=Vid*1000),
                     G3=list(V=diag(4), nu=4, alpha.mu=rep(0,4), alpha.V=VM*1000),
                     G4=list(V=diag(3), nu=3, alpha.mu=rep(0,3), alpha.V=VY*1000)),
              R=list(V=VR, nu=4))

mgLRS0<-MCMCglmm(cbind(Weight,Body_Length,Tail_Length,rLRS) ~ trait-1 + trait:(f)+at.level(trait,c(1:3)):(Sex*Age*StDate+StDateSq),
                 random =~ us(trait):animal + us(trait):id + us(trait):Mother + idh(at.level(trait,c(1:3))):Year, 
                 rcov=~us(trait):units, prior = priorg0,start=startg0, data=AllM,pedigree=ped, family=c("gaussian","gaussian","gaussian","gaussian"),
                 verbose=TRUE,nitt=1300000,burnin=300000,thin=1000,pr=FALSE)


#### Robertson-Price decomposition for fitness components ####
#Adult males ARS
priorARSad <- list(G=list(G1=list(V=diag(2), nu=2),
                        G2=list(V=diag(2), nu=2),
                        G3=list(V=diag(2), nu=2),
                        G4=list(V=diag(2), nu=2)),
                 R=list(V=diag(2),nu=2))


mARSadMNorm <- MCMCglmm(cbind(Weight,rARS)~trait-1 + at.level(trait,1):(StDate+StDateSq),
                      random=~us(trait):animal+us(trait):id+us(trait):Mother+idh(trait):Year, rcov=~us(trait):units,
                      pedigree=ped,data=AllM[which( AllM$Age=="A" & AllM$Sex=="Male" ),], 
                      prior = priorARSad,family=c("gaussian","gaussian"),verbose=TRUE,nitt=110000,burnin=10000,thin=100)
summary(mARSadMNorm)

#Adult females ARS
mARSadFNorm<-MCMCglmm(cbind(Weight,rARS)~trait-1 + at.level(trait,1):(StDate+StDateSq),
                      random=~us(trait):animal+us(trait):id+us(trait):Mother+idh(trait):Year, rcov=~us(trait):units,
                      pedigree=ped,data=AllM[which( AllM$Age=="A" & AllM$Sex=="Female" ),], 
                      prior = priorARSad,family=c("gaussian","gaussian"),verbose=TRUE,nitt=110000,burnin=10000,thin=100)
summary(mARSadFNorm)


#Juvenile survival
priorPhimadNorm<-list(G=list(G1=list(V=diag(2), nu=2),
                             G2=list(V=diag(1), nu=1),
                             G3=list(V=diag(2), nu=2),
                             G4=list(V=diag(2), nu=2)),
                      R=list(V=diag(2),nu=2,fix=2))

mRelphimjuvNorm<-MCMCglmm(cbind(Weight,rPhi)~trait-1 + at.level(trait,1):(Sex*StDate+StDateSq+Age3+f)+ at.level(trait,2):(Sex+f),
                          random=~us(trait):animal+us(at.level(trait,1)):id+us(trait):Mother+idh(trait):Year, rcov=~us(trait):units,
                          pedigree=ped,data=AllM[which( AllM$Age=="J"),],
                          prior = priorPhimadNorm,family=c("gaussian","gaussian"),verbose=TRUE,nitt=110000,burnin=10000,thin=100)
summary(mRelphimjuvNorm)

#Adult survival
mRelphimadNorm<-MCMCglmm(cbind(Weight,rPhi)~trait-1 + at.level(trait,1):(Sex*StDate+StDateSq+Age3+f)+ at.level(trait,2):(Sex+f),
                         random=~us(trait):animal+us(at.level(trait,1)):id+us(trait):Mother+idh(trait):Year, rcov=~us(trait):units,
                         pedigree=ped,data=AllM[which(!duplicated(paste(AllM$id,AllM$Year)) & AllM$Age=="A"),],
                         prior = priorPhimadNorm,family=c("gaussian","gaussian"),verbose=TRUE,nitt=110000,burnin=10000,thin=100)
summary(mRelphimadNorm)



########################################################
########################################################

##### GROWTH SURVIVAL ANALYSIS #####
library(R2jags)
library(MCMCglmm)

## Function to create initial values
##############################################%%
F_initPlsparse<-function(MaxLitter=4,mums=mums,mother=mother){
  mothernb<-length(mums)
  pld<-matrix(data=0,nrow=nind,ncol=MaxLitter)
  pli<-matrix(data=NA,nrow=nind,ncol=MaxLitter)
  
  for(m in 1:mothernb)#this assumes that the first individual must come from the first litter!! (as it should be indeed now that data are re-ordered)
  {
    ind<-which(mother==m)
    for (i in 1:length(ind))
    {
      if (i==1)
      {
        pld[ind[i],1:MaxLitter]<-c(1,rep(x=0,times=MaxLitter-1))
        pli[ind[i],1:MaxLitter]<-NA
      }else{
        pld[ind[i],1:MaxLitter]<-NA
        pli[ind[i],1:MaxLitter]<-runif(MaxLitter,0,1)
      }
    }
  }
  
  initmeanLB<-matrix(data = 0,nrow = mothernb,ncol = MaxLitter)
  
  for (i in 1:nrow(initmeanLB))
  {
    initmeanLB[i,1]<-runif(1,0,40)
    for (j in 2: ncol(initmeanLB))
    {
      initmeanLB[i,j]<-runif(1,initmeanLB[i,j-1]+20,initmeanLB[i,j-1]+60)
    }
  }
  toreturn<-list(pld=pld,pli=pli,initmeanLB=initmeanLB)
  return(toreturn)
}

### JAGS model ###
##############################
sink("models/GrowthViability")
cat("
    model {
    ###################################
    ######priors and constraints#######
    ###################################
    #####survival######
    meanmu~dnorm(0,0.001)
    mean.phi<-1/(1+exp(-meanmu))#logit-1 transform
    BetaA~dnorm(0,0.001)#selection coefficient for asymptotic size
    BetaD~dnorm(0,0.001)#selection coefficient for time to first snow fall
    BetaAD~dnorm(0,0.001)#selection coefficient for interaction asymptotic size and time to first snow fall
    BetaS~dnorm(0,0.001)#sex difference in survival
    
    #####growth#####
    for (i in 1:nind)#loop over individuals
    {
    A[i]~dunif(mA-5,mA+16)#individual asymptotic size
    k[i]~dnorm(mk,tauk)T(0.01,0.025) #individual growth rate
    }
    tauk<-pow(sdk,-2)#precision of individual growth rates
    sdk~dunif(0,0.001)#standard deviation of individual growth rates
    
    tau<-pow(sdMass,-2)#precision in mass measurment
    sdMass<-2.05#standard deviation in mass measurment, estimated from animals measured multiple times
    
    ## Birth dates
    for (mum in 1:mothernb)#loop over mothers*year
    {
    #first litter
    meanLB[mum,1]~dunif(-30,maxbd)  #date of first litter of the mother on this year (between -30, i.e. April 26th and October 5th)
    for (i in 2:MaxLitter)#loop over litters 2 to 5
    {
    meanLB[mum,i]~dunif(gbegin[mum,i],gend[mum,i])#date of successive litters
    gbegin[mum,i]<-meanLB[mum,i-1]+20 #with a minimum of 20 days between successive litters...
    gend[mum,i]<-meanLB[mum,i-1]+120#and a maximum of 120 days
    }
    }
    for (i in 1:nind)
    {
    for(j in 1:MaxLitter)
    {
    pl[i,j]~dgamma(1,1)
    plG[i,j]<-pl[i,j]/sum(pl[i,])
    }
    }
    ###################################
    ############likelihood#############
    ###################################
    for (obs in 1:nobs)
    {
    M[obs]~dnorm(mf[obs],tau)
    mf[obs]<-A[whichind[obs]]*(1-exp(-k[whichind[obs]]*(date[obs])))
    date[obs]<-t[obs]-meanLB[mother[whichind[obs]],litter[whichind[obs]]]
    }
    for (i in 1:nind)
    {
    DeltaWinter[i]<-endSeason[IndCohort[i]]-meanLB[mother[i],litter[i]]
    litter[i] ~ dcat(plG[i,])
    phi[i]~dbin(p[i],1)
    logit(p[i])<-meanmu+BetaA*A[i]+BetaD*DeltaWinter[i]+BetaAD*A[i]*DeltaWinter[i]+BetaS*sex[i]
    pa[i]<-phi[i]*A[i]
    }    
    }
    ",fill = TRUE)
sink()


sparceinitPl<-F_initPlsparse(MaxLitter = MaxLitter,mums = mums,mother = mother)
pld<-sparceinitPl$pld
pli<-sparceinitPl$pli
initmeanLB<-sparceinitPl$initmeanLB

dataGrowthAKlphi2<-list(mothernb=mothernb,M=mass$Weight,endSeason=endSeason,
                        t=mass$RelativeJulian,IndCohort=IndCohort,sex=sex,
                        nind=length(unique(mass$id)),nobs=length(mass$Weight),
                        pl=pld,MaxLitter=MaxLitter,mother=mother,
                        whichind=whichind,maxbd=maxbd,mA=mA,mk=mk,phi=phis,A=Aobs)
initsGAKlphi2 <- function() list(A=initA,k=ki,pl=pli,meanLB=initmeanLB,BetaA=0,BetaD=0,dBetaA=0,BetaS=0,meanmu=0,mu=0.5,tauphi2=0.1)

paramsGAKlphi2 <- c("A","k","litter","meanLB","BetaA","BetaD","BetaAD","meanmu","BetaS","mf","p")
# MCMC settings

GrowthAKlphi2b<-jags(dataGrowthAKlphi2,initsGAKlphi2,paramsGAKlphi2,"models/GrowthViability",
                     n.chains = 3, n.thin = 6000, n.iter = 6300000, n.burnin = 300000, working.directory = getwd())
sumGAK2b<-print(GrowthAKlphi2b)
mean(sumGAK2b$summary[,"Rhat"])
sumGAK2b$summary[which(sumGAK2b$summary[,"Rhat"]>1.1),"Rhat"]

traceplot(GrowthAKlphi2b ,varname="deviance")

GrowthAKlphi2c<-jags(dataGrowthAKlphi2,initsGAKlphi2,paramsGAKlphi2,"models/GrowthViability",
                     n.chains = 3, n.thin = 3000, n.iter = 3300000, n.burnin = 300000, working.directory = getwd())


######################## Posterior Predictive Checks
ppcmean<-vector(length = 3000)
count<-1
for (i in 1:1000)
  for(j in 1:3)
  {
    ppcmean[count]<-mean(mass$Weight) - mean(GrowthAKlphi2c$BUGSoutput$sims.array[i,j,3000+(1:1225)])
    count<-count+1
  }
hist(ppcmean)
mean(ppcmean>0)# posterior predictive check for mean mass p-value= 0.794


ppc<-vector(length = 3000*length(mass$Weight))
count<-1
for (ind in 1: length(mass$Weight))
  for (i in 1: 1000)
    for (j in 1:3)
    {
      ppc[count]<-mass$Weight[ind]-GrowthAKlphi2c$BUGSoutput$sims.array[i,j,3000+ind] 
      count<-count+1
    }

plot(ppc[1:10000])
mean(ppc>0)#posterior predictive check p-value for mass 0.459

names(GrowthAKlphi2c$BUGSoutput$sims.array[1,1,3000:4838])

ppcmeanPhi<-vector(length = 3000)
count<-1
for (i in 1:1000)
  for(j in 1:3)
  {
    ppcmeanPhi[count]<-mean(phis) - mean(GrowthAKlphi2c$BUGSoutput$sims.array[i,j,3000+(1227:1838)])
    count<-count+1
  }
hist(ppcmeanPhi)
mean(ppcmeanPhi>0)# posterior predictive check for mean survival p-value= 0.499